#!perl

use Test::More tests => 16;
use Trace;
my $STDOUT;

ok(1, "partial credit for showing up");

my $F = Fruit->new;
my $x = Trace->new($F);
ok($x, "Trace object returned");

sub divert () { $STDOUT = ""; tie *STDOUT => Divert, \$STDOUT or die; }
sub undivert () { untie *STDOUT }

divert;
$x->set("color", "red");
undivert;
is($STDOUT, "Fruit->set(color, red) called on object $F");

divert;
my $res = $x->get("color");
undivert;
is($STDOUT, "Fruit->get(color) called on object $F");
is($res, "red", "return value");

divert;
$x->sideeffect("carrots");
undivert;
is($STDOUT, "Fruit->sideeffect(carrots) called on object $F");
is(Fruit->V, "carrots", "side effect");

divert;
$x->no_args;
undivert;
is($STDOUT, "Fruit->no_args() called on object $F");
is(Fruit->V, "$F", "object passed OK");

divert;
$res = $x->V(1..5);
undivert;
is($STDOUT, "Fruit->V(1, 2, 3, 4, 5) called on object $F");
is($res, "$F", "object passed OK again");

divert;
my $F2 = $x->new();
undivert;
is($STDOUT, "Fruit->new() called on object $F");
is(ref($F2), "Fruit", "created a fruit");
is($F2->get("color"), "red", "created red fruit");

divert;
my $y = Trace->new(bless {} , "Whoops!");
undef $y;
undivert;
is($STDOUT, "", "DESTROY was quiet");

divert;
$x->nosuchmethod;
undivert;
is($STDOUT, "", "nosuchmethod was quiet");

package Divert;

sub TIEHANDLE {
  my ($class, $sink) = @_;
  bless { SINK => $sink } => $class;
}

sub PRINT {
  my ($self, $data) = @_;
  $data =~ s/\n$//;
  $ {$self->{SINK}} .= $data;
}

package Fruit;

my $V;

sub new {
  my $base = shift;
  my $class = ref($base) || $base;
  my $self = ref($base) ? {%$base} : {};
  bless $self => $class;
}

sub set {
  my ($self, $k, $v) = @_;
  $self->{$k} = $v;
}

sub get {
  my ($self, $k) = @_;
  return $self->{$k};
}

sub sideeffect {
  my ($self, $v) = @_;
  $V = $v;
}

sub no_args {
  my $self = shift;
  $V = $self;
}

sub V { $V }

